#include "CovDeterminant.h"
#include "Eigen/Dense"
#include "Eigen/Eigenvalues"
using namespace Eigen;

CovDet::CovDet()
{
	cov.setZero();
	area = 0;
	centroid = Vector3d(0,0,0);
}

CovDet::CovDet(Vector3d v1_coor, Vector3d v2_coor, Vector3d v3_coor)
{
	area = 0.5 * (v2_coor-v1_coor).cross(v3_coor-v1_coor).norm();
	centroid = 1/3.0 * (v1_coor + v2_coor + v3_coor);
	Matrix3d vm, c;
	vm(0,0) = v1_coor(0);
	vm(1,0) = v1_coor(1);
	vm(2,0) = v1_coor(2);
	vm(0,1) = v2_coor(0);
	vm(1,1) = v2_coor(1);
	vm(2,1) = v2_coor(2);
	vm(0,2) = v3_coor(0);
	vm(1,2) = v3_coor(1);
	vm(2,2) = v3_coor(2);

	c(0,0) = c(1,1) = c(2,2) = 2;
	c(0,1) = c(0,2) = c(1,0) = c(1,2) = c(2,0) = c(2,1) = -1;

	cov = area/36.0 * vm * c * vm.transpose();
}

double CovDet::energy()
{
	if (cov.determinant()/pow(area, 5) < 1e-17)
	{
		return cov.trace()*area*1e-20;
	}
	else
		return cov.determinant()/pow(area, 4);
}


vec3 CovDet::getPlaneNormal()
{
	SelfAdjointEigenSolver<Matrix3d> es(cov);
	int smallest = 0;

	for (int i=1; i<3; i++)
	{
		if (abs(es.eigenvalues()[i]) < abs(es.eigenvalues()[smallest]))
			smallest = i;
	}
	const Vector3d res = es.eigenvectors().col(smallest);

	return vec3(res(0), res(1), res(2));
}