#ifndef COV_DETERMINANT_H
#define COV_DETERMINANT_H
#include "Eigen/Dense"
#include "Eigen/Eigenvalues"
using namespace Eigen;

#include "Vector3.h"
class CovDet
{
public:
	Matrix3d cov;
	double area;
	Vector3d centroid;
public:
	EIGEN_MAKE_ALIGNED_OPERATOR_NEW
public:
	CovDet();
	CovDet(Vector3d v1_coor, Vector3d v2_coor, Vector3d v3_coor);
	
	CovDet& operator=(const CovDet& Q)
	{ cov = Q.cov; area = Q.area; centroid = Q.centroid; return *this; }
	CovDet& operator+=(const CovDet& Q)
	{ 
		//some models may contain corrupted null faces, ignore them
		if (Q.area < 1e-18)
			return *this;
		CovDet Q_old = *this;
		area = Q_old.area + Q.area;
		centroid = (Q_old.area * Q_old.centroid + Q.area * Q.centroid)/(area);
		Vector3d ktoi, ktoj;
		ktoi = Q_old.centroid - centroid;
		ktoj = Q.centroid - centroid;
		cov = Q_old.cov + Q.cov + Q_old.area * ktoi * ktoi.transpose() + Q.area * ktoj * ktoj.transpose(); 
		return *this; 
	}

	CovDet& operator-=(const CovDet& Q)
	{ 
		if (Q.area < 1e-18)
			return *this;
		CovDet Q_old = *this;
		area = Q_old.area - Q.area;
		centroid = (Q_old.area * Q_old.centroid - Q.area * Q.centroid)/(area);
		Vector3d ktoi, ktoj;
		ktoi = Q_old.centroid - centroid;
		ktoj = Q_old.centroid - Q.centroid;
		cov = Q_old.cov - Q.cov - area * ktoi * ktoi.transpose() - Q.area * ktoj * ktoj.transpose(); 
		return *this; 
	}

	double energy();
	vec3 getPlaneNormal();
};

#endif
