#ifndef DIJKSTRA_H
#define DIJKSTRA_H
#include "MxStdSlim.h"
#include <vector>
#include <set>
#include <iterator>
#include <algorithm>

class PathInfo : public MxHeapable
{
public:
	int node_id;
	float dist;

	mutable int prev;
};  

class PathCompare
{
public:
	bool operator()(const PathInfo p1, const PathInfo p2)
	{
		if(p1.dist != p2.dist)
			return p1.dist < p2.dist;
		else
			return p1.node_id < p2.node_id;
	}
};

class Dijkstra
{
public:
	Dijkstra(int nclusters, int node_size);
	void initialize_neighbors();
	void initialize_boundary_flooding(int cluster_id, vector<pair<int,int>> source_and_scalars);
	void flood_cluster_boundary(int cluster_id);
	void flood_cluster_inner(int cluster_id);

protected:
	double get_edge_length(int node_i, int node_j);
	void heapify_boundary_neighbors(int cluster_id, int node_id, bool source);
	void heapify_inner_neighbors(int cluster_id, int node_id, bool source);
public:
	vector<set<int>> cluster_nodes;
	vector<int> nearest_scalar;
	vector<bool> flooded;
	vector<bool> propagated;
	
	vector<set<int>> neighbor_nodes;
	vector<set<int>> boundary_neighbor_nodes;
	vector<set<int>> inner_neighbor_nodes;

	set<PathInfo, PathCompare> available_paths;
	vector<float> closest_dist;
};
#endif