#include "mesh.h"

void Mesh::initMeshInfo()
{
	vector<int> s;
	face_neighbors = vector<vector<int>>(mesh_face.size(), s);
	node_neighbors = vector<vector<int>>(mesh_node.size(), s);
	vector<Hedge> * node_hedges = new vector<Hedge>[mesh_node.size()];
	for (int i=0; i<mesh_face.size(); i++)
	{
		int v1 = mesh_face[i].x;
		int v2 = mesh_face[i].y;
		int v3 = mesh_face[i].z;

		node_hedges[v1].push_back(Hedge(v1,v2,i));
		node_hedges[v2].push_back(Hedge(v2,v3,i));
		node_hedges[v3].push_back(Hedge(v3,v1,i));

		node_neighbors[v1].push_back(i);
		node_neighbors[v2].push_back(i);
		node_neighbors[v3].push_back(i);
	}

	for (int i=0; i<mesh_face.size(); i++)
	{
		int v1 = mesh_face[i].x;
		int v2 = mesh_face[i].y;
		int v3 = mesh_face[i].z;

		//check v2 -> v1
		vector<Hedge> hedges = node_hedges[v2];
		bool reverse_exist = false;
		for (vector<Hedge>::iterator it = hedges.begin(); it != hedges.end(); it++)
		{
			if (it->toNode == v1)
			{
				face_neighbors[i].push_back(it->face);
				reverse_exist = true;
			}
		}
		if (!reverse_exist)
		{
			mesh_node[v1].boundary_node = true;
			mesh_node[v2].boundary_node = true;
		}

		//check v3 -> v2
		hedges = node_hedges[v3];
		reverse_exist = false;
		for (vector<Hedge>::iterator it = hedges.begin(); it != hedges.end(); it++)
		{
			if (it->toNode == v2)
			{
				face_neighbors[i].push_back(it->face);
				reverse_exist = true;
			}
		}
		if (!reverse_exist)
		{
			mesh_node[v2].boundary_node = true;
			mesh_node[v3].boundary_node = true;
		}

		//check v1 -> v3
		hedges = node_hedges[v1];
		reverse_exist = false;
		for (vector<Hedge>::iterator it = hedges.begin(); it != hedges.end(); it++)
		{
			if (it->toNode == v3)
			{
				face_neighbors[i].push_back(it->face);
				reverse_exist = true;
			}
		}
		if (!reverse_exist)
		{
			mesh_node[v1].boundary_node = true;
			mesh_node[v3].boundary_node = true;
		}
	}

	delete [] node_hedges;
}