#include "pca_remeshing.h"
#include <fstream>
#include "mesh.h"
#include "partition.h"
#include "remesh.h"
#include "Vector3.h"
#define MAX_ITER 100

extern string in_file;
extern int valid_clusters;
extern Mesh init_mesh;
extern Mesh contracted_mesh;
extern Partition* mesh_partition;
extern Remesh* mh;

PCA_Remeshing::PCA_Remeshing()
{
	contracted_num = 100;
}

PCA_Remeshing::~PCA_Remeshing()
{

}

void PCA_Remeshing::readOffFile(string in_file)
{
	in_file = in_file;
	ifstream ip(in_file.c_str());
	string file_ext;
	long int np;
	long int nf;
	int extend_int;

	ip >> file_ext >> np >> nf >> extend_int;
	double x, y, z;
	double nx, ny, nz;
	for (int i=0; i<np; i++)
	{
		ip >> x >> y >> z;

		init_mesh.mesh_node.push_back(Node(x,y,z));
		if (x > init_mesh.xmax)
			init_mesh.xmax = x;
		else if (x < init_mesh.xmin)
			init_mesh.xmin = x;
		if (y > init_mesh.ymax)
			init_mesh.ymax = y;
		else if (y < init_mesh.ymin)
			init_mesh.ymin = y;
		if (z > init_mesh.zmax)
			init_mesh.zmax = z;
		else if (z < init_mesh.zmin)
			init_mesh.zmin = z;
	}

	long int v1, v2, v3;
	int np_face;
	for (int i=0; i< nf; i++)
	{
		ip >> np_face >> v1 >> v2 >> v3;
		assert(np_face == 3);
		assert(v1 < np);
		assert(v2 < np);
		assert(v3 < np);
		init_mesh.mesh_face.push_back(Vector3<long int>(v1, v2, v3));
	}

	assert(init_mesh.mesh_face.size() == nf);
	assert(init_mesh.mesh_node.size() == np);
	contracted_mesh = init_mesh;
	init_mesh.init_alive_face();
	contracted_mesh.init_alive_face();
	ip.close();

	cout << "read model from " << in_file.c_str() << endl;
	cout << "vertex number: " << np <<  endl;
	cout << "face number: " << nf <<  endl;
}

void PCA_Remeshing::init()
{
	if (mesh_partition != NULL)
		delete mesh_partition;
	mesh_partition = new Partition();
	cout << "start merging" << endl;
	mesh_partition->init_cov();
}

void PCA_Remeshing::decimate()
{
	if (mesh_partition != NULL)
	{
		mesh_partition->decimate(contracted_num);
		cout << "merging complete!" << endl;
	}
}

void PCA_Remeshing::init_swap()
{
	if (mesh_partition != NULL)
		mesh_partition->init_face_swap();
}

void PCA_Remeshing::swap_once()
{
	if (mesh_partition != NULL)
		mesh_partition->swap_face_once();
}

void PCA_Remeshing::swap_always()
{
	if (mesh_partition != NULL)
	{
		cout << "start swapping" << endl;
		int iter = 0;
		while(mesh_partition->swap_face_once() && iter <= MAX_ITER)
			iter++;
		cout << endl << "swapping complete!" << endl;

		mesh_partition->output_partition_ply();
	}	
}


void PCA_Remeshing::generate_poly_mesh()
{
	if (mesh_partition != NULL)
	{
		mh = new Remesh();
		mh->cleanClusters();
		mh->construct_poly_mesh();
	}
}

void PCA_Remeshing::generate_CDT_mesh()
{
	if (mesh_partition != NULL && mh->poly_mesh_ready)
	{
		mh->construct_CDT_mesh();
	}
}
